#ifdef _WIN32
#include "windows.h"
#else
#include <stddef.h>
#include <sys/types.h>
#include <dirent.h>
#include "ctype.h"
#include "unistd.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "auxiliar.h"

#include "BList.h"
#include "BWindow.h"

#include "BFileListWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BFileListWnd::BFileListWnd(int x,int y,int dx,int dy,char *patt,char *pck,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	old_pattern[0]=0;
	pattern=patt;
	pick=pck;

} /* BFileListWnd::BFileListWnd */ 


BFileListWnd::~BFileListWnd()
{
} /* BFileListWnd::~BFileListWnd */ 


bool BFileListWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (strcmp(old_pattern,pattern)!=0) {
		dirty=true;
#ifdef _WIN32
		/* Look for files: */ 
		WIN32_FIND_DATA finfo;
		HANDLE h;
		char tmp[256],tmp2[256];

		files.Delete();

		{
			int i=0,last=-1;
			strcpy(tmp,pattern);

			while(tmp[i]!=0) {
				if (tmp[i]=='\\' || tmp[i]=='/') {
					last=i;
				} /* if */ 
				i++;
			} /* while */ 
			tmp[last+1]=0;
			strcat(tmp,"*.*");
		}

		h=FindFirstFile(tmp,&finfo);
		if (h!=INVALID_HANDLE_VALUE) {
			if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0) {
				char *tmp;

				tmp=new char[strlen(finfo.cFileName)+2];
				tmp[0]='/';
				strcpy(tmp+1,finfo.cFileName);
				files.Add(tmp);
			} /* if */ 

			while(FindNextFile(h,&finfo)==TRUE) {
				if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0) {
					char *tmp;

				tmp=new char[strlen(finfo.cFileName)+2];
				tmp[0]='/';
				strcpy(tmp+1,finfo.cFileName);			
				files.Add(tmp);
				} /* if */ 
			} /* while */ 
		} /* if */ 
		FindClose(h);
                
		/* Obtain the extension of the "pattern": */ 
		{
			int i,j,k;

			strcpy(tmp2,pattern);
			
			for(i=0,j=-1;tmp2[i]!=0;i++) if (tmp2[i]=='\\' || tmp2[i]=='/') j=i;
			tmp2[j+1]=0;

			j++;
			for(i=j,k=j;pattern[i]!=0;i++) {
				if (pattern[i]=='.') k=i;
			} /* if */ 

			if (k!=j) {
				strcat(tmp2,"*");
				strcat(tmp2,pattern+k);
			} else {
				strcpy(tmp2,pattern);
			} /* if */ 
			
		}
                
		h=FindFirstFile(tmp2,&finfo);
		if (h!=INVALID_HANDLE_VALUE) {
			if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)==0) {
				char *tmp;

				tmp=new char[strlen(finfo.cFileName)+1];
				strcpy(tmp,finfo.cFileName);
				files.Add(tmp);
			} /* if */ 

			while(FindNextFile(h,&finfo)==TRUE) {
				if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)==0) {
					char *tmp;

					tmp=new char[strlen(finfo.cFileName)+1];
					strcpy(tmp,finfo.cFileName);
					files.Add(tmp);
				} /* if */ 
			} /* while */ 
		} /* if */ 

		FindClose(h);
                
#else
		char tmp[256],tmp2[256];
		DIR *dp;
		struct dirent *ep;

		files.Delete();

		{
			int i=0,j=-1,k;
			strcpy(tmp,pattern);

			for(i=0,j=-1;tmp[i]!=0;i++) if (tmp[i]=='\\' || tmp[i]=='/') j=i;
			tmp[j]=0;

			j++;
			for(i=j,k=j;tmp[i]!=0;i++) if (tmp[i]=='.') k=i;
			
			if (k!=j) {
				strcat(tmp2,pattern+k);
			} else {
                                tmp2[0]=0;
			} /* if */ 
		}
                               
		dp = opendir (tmp);
		if (dp != NULL) {
		    while (ep = readdir (dp)) {
				if (ep->d_type==DT_DIR) {
					char *tmp;

					tmp=new char[strlen(ep->d_name)+2];
					tmp[0]='/';
					strcpy(tmp+1,ep->d_name);
					files.Add(tmp);
				} /* if */ 
            } /* while */ 
            (void) closedir (dp);
        } /* if */ 
		dp = opendir (tmp);
		if (dp != NULL) {
		    while (ep = readdir (dp)) {
				if (ep->d_type!=DT_DIR) {
					if (strlen(tmp2)<=strlen(ep->d_name) &&
						strcmp(tmp2,ep->d_name+strlen(ep->d_name)-strlen(tmp2))==0) {
						char *tmp;

						tmp=new char[strlen(ep->d_name)+1];
						strcpy(tmp,ep->d_name);
						files.Add(tmp);
					} /* if */ 
				} /* if */ 
            } /* while */ 
            (void) closedir (dp);
        } /* if */ 
                                  
#endif

                strcpy(old_pattern,pattern);

	} /* if */ 

	if (mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1)) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<char> l;
		char *f;
		int y=2;

		l.Instance(files);
		l.Rewind();
		while(l.Iterate(f)) {
			if (local_mouse_y>=y && 
				local_mouse_y<=y+TTF_FontHeight(font) &&
				local_mouse_x>=2 &&
				local_mouse_x<=int(2+get_text_width(f,font))) {	
				if (f[0]=='/') {
					strcpy(pick,f+1);
				} else {
					strcpy(pick,f);
				} /* if */ 
			} /* if */ 
			y+=2+TTF_FontHeight(font);
		} /* while */ 
	} /* if */ 

	return true;
} /* BFileListWnd::cycle */ 


void BFileListWnd::contens_size(int *dx,int *dy)
{
	int max_y;

	*dx=old_dx;
	*dy=old_dy;

	max_y=2+files.Length()*(TTF_FontHeight(font)+2);

	if (max_y>*dy) *dy=max_y;
} /* BFileListWnd::contens_size */ 


void BFileListWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	char *tmp;
	List<char> l;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	y=2;
	l.Instance(files);
	l.Rewind();
	while(l.Iterate(tmp)) {
		if (tmp[0]=='/') {
			print_left(tmp+1,font,128,128,255,sfc,2-stx,y-sty);
		} else {
			print_left(tmp,font,255,255,255,sfc,2-stx,y-sty);
		} /* if */ 
		y+=2+TTF_FontHeight(font);
	} /* while */ 

} /* BFileListWnd::draw_contens */ 

